from pathlib import Path
from typing import Optional, Union

from attrs import define, field


@define(frozen=True, slots=True, kw_only=True)
class Badge:
    name: str
    description: str
    example: str

    title: str
    link: str
    image: str

    weight: int = 0


@define(frozen=True, slots=True, kw_only=True)
class Node:
    tokens: set[str]


@define(frozen=True, slots=True, kw_only=True)
class File(Node):
    path: Path
    pattern: str


@define(frozen=True, slots=True, kw_only=True)
class OldRemote:
    name: str
    prefix: str


@define(frozen=True, slots=True, kw_only=True)
class RemoteMatch:
    host: str
    path_prefix: Optional[str] = None


@define(frozen=True, slots=True, kw_only=True)
class Remote(Node):
    host: str
    path: Optional[str] = None


@define(frozen=True, slots=True, kw_only=True)
class ProjectRemote:
    name: str
    type: str
    url: str
    scheme: Union[str, None]
    user: Union[str, None]
    host: str
    path: str


@define(frozen=True, slots=True, kw_only=True)
class Project:
    local_path: Path
    url: str
    ref: str
    name: str
    namespace: str
    path: str
    full_name: str
    full_path: str

    remotes: dict[str, dict[str, ProjectRemote]] = field(factory=dict)


@define(frozen=True, slots=True, kw_only=True)
class GitLabProject(Node):
    url: str
    ref: str
    namespace: str
    name: str
    full_name: str
    path: str
    full_path: str


@define(frozen=True, slots=True, kw_only=True)
class Hook(Node):
    repo: str
    hook: str


@define(frozen=True, slots=True, kw_only=True)
class HookMatch:
    repo: str
    hook: str


@define(slots=True, kw_only=True)
class Context:
    path: Path

    nodes: dict[str, list[Node]] = field(factory=dict)
    tokens_found: set[Node] = field(factory=set)
    tokens_processed: set[Node] = field(factory=set)

    providers: dict[Node] = field(factory=dict)
    badges: dict[Node] = field(factory=dict)

    def add_nodes(self, nodes):
        for node in nodes:
            for token in node.tokens:
                self.nodes.setdefault(token, [])
                self.nodes[token].append(node)
            self.tokens_found |= node.tokens

    def run(self, module):
        nodes = module.run(self)
        self.add_nodes(nodes)



from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.PYTHON_BLACK: Badge(
            name="code-style-black",
            description="Show the Black code style.",
            example="https://img.shields.io/badge/code_style-black-000000.svg",
            title="code style: black",
            link="https://github.com/psf/black",
            image="https://img.shields.io/badge/code_style-black-000000.svg",
            weight=20,
        ),
        to.PRETTIER: Badge(
            name="code-style-prettier",
            description="Show the Prettier code style badge.",
            example="https://img.shields.io/badge/code_style-prettier-ff69b4.svg",
            title="code style: prettier",
            link="https://github.com/prettier/prettier",
            image="https://img.shields.io/badge/code_style-prettier-ff69b4.svg",
            weight=20,
        ),
    }
)



from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.GITLAB_COVERAGE: Badge(
            name="gitlab-coverage-report",
            description="Show the most recent coverage score on the default branch.",
            example="https://img.shields.io/gitlab/pipeline-coverage/brettops/tools/badgie",
            title="coverage report",
            link="{node.url}/-/commits/{node.ref}",
            image="https://img.shields.io/gitlab/pipeline-coverage/{node.full_path}",
            weight=1,
        ),
        to.GITLAB_PIPELINE: Badge(
            name="gitlab-pipeline-status",
            description="Show the most recent pipeline status on the default branch.",
            example="https://img.shields.io/gitlab/pipeline-status/brettops/tools/badgie",
            title="pipeline status",
            link="{node.url}/-/commits/{node.ref}",
            image="https://img.shields.io/gitlab/pipeline-status/{node.full_path}",
            weight=0,
        ),
        to.GITLAB_RELEASE: Badge(
            name="gitlab-latest-release",
            description="Show the latest GitLab release by date.",
            example="https://img.shields.io/gitlab/v/release/brettops/tools/badgie",
            title="latest release",
            link="{node.url}/-/releases",
            image="https://img.shields.io/gitlab/v/release/{node.full_path}",
            weight=2,
        ),
    }
)



ANSIBLE = "ansible"
ANSIBLE_COLLECTION = "ansible_collection"
ANSIBLE_GALAXY = "ansible_galaxy"
ANSIBLE_LINT = "ansible_lint"
ANSIBLE_ROLE = "ansible_role"

BRETTOPS = "brettops"
BRETTOPS_CONTAINER = "brettops_container"
BRETTOPS_PACKAGE = "brettops_package"
BRETTOPS_PIPELINE = "brettops_pipeline"
BRETTOPS_ROLE = "brettops_role"
BRETTOPS_TOOL = "brettops_tool"

DETECT_SECRETS = "detect_secrets"  # pragma: allowlist secret

GIT = "git"

GITHUB = "github"
GITHUB_ACTIONS = "github_actions"

GITLAB = "gitlab"
GITLAB_CI_FILE = "gitlab_ci_file"
GITLAB_COVERAGE = "gitlab_coverage"
GITLAB_ENVIRONMENT = "gitlab_environment"
GITLAB_PAGES = "gitlab_pages"
GITLAB_PIPELINE = "gitlab_pipeline"
GITLAB_RELEASE = "gitlab_release"

PRE_COMMIT = "pre_commit"
PRE_COMMIT_CONFIG = "pre_commit_config"
PRE_COMMIT_HOOKS = "pre_commit_hooks"

PRETTIER = "prettier"

PYPI_DOWNLOADS = "pypi_downloads"
PYPI_FORMAT = "pypi_format"
PYPI_LICENSE = "pypi_license"
PYPI_PYTHON_VERSION = "pypi_python_version"
PYPI_STATUS = "pypi_status"
PYPI_VERSION = "pypi_version"
PYPI_WHEEL = "pypi_wheel"

PYTHON = "python"
PYTHON_BLACK = "python_black"
PYTHON_IMPLEMENTATION = "python_implementation"
PYTHON_ISORT = "python_isort"
PYTHON_PYPROJECT_TOML = "python_pyproject_toml"
PYTHON_SETUP_CFG = "python_setup_cfg"
PYTHON_SETUPTOOLS = "python_setuptools"

SHELL_SHFMT = "shell_shfmt"

TERRAFORM = "terraform"
TERRAFORM_MODULE = "terraform_module"
TERRAFORM_PROVIDER = "terraform_provider"
TERRAFORM_WORKSPACE = "terraform_workspace"

# full list of tokens

TOKENS = [token for token in dict(globals()).items() if not token[0].startswith("_")]
